==> js-20.0.0-sources.jar: com/oracle/truffle/js/nodes/unary/IsIdenticalIntegerNode.java <==
/*
 * Copyright (c) 2018, 2018, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.js.nodes.unary;

import com.oracle.truffle.api.dsl.Fallback;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.js.nodes.JavaScriptNode;
import com.oracle.truffle.js.runtime.BigInt;
import com.oracle.truffle.js.runtime.JSRuntime;

public abstract class IsIdenticalIntegerNode extends IsIdenticalBaseNode {

    private final int integer;

    protected IsIdenticalIntegerNode(JavaScriptNode operand, int integer, boolean leftConstant) {
        super(operand, leftConstant);
        this.integer = integer;
    }

    @Specialization
    protected boolean doInt(int a) {
        return a == integer;
    }

    @Specialization
    protected boolean doDouble(double a) {
        return a == integer;
    }

    @Specialization
    protected boolean doBigInt(@SuppressWarnings("unused") BigInt a) {
        return false;
    }

    // long etc could come via Interop
    @Specialization(guards = "isJavaNumber(a)")
    protected boolean doJavaNumber(Object a) {
        double doubleValue = JSRuntime.toDouble(a);
        return doubleValue == integer;
    }

    @Fallback
    protected boolean doOther(@SuppressWarnings("unused") Object other) {
        return false;
    }

    public static IsIdenticalIntegerNode create(int integer, JavaScriptNode operand, boolean leftConstant) {
        return IsIdenticalIntegerNodeGen.create(operand, integer, leftConstant);
    }

    @Override
    protected JavaScriptNode copyUninitialized() {
        return create(integer, cloneUninitialized(getOperand()), leftConstant);
    }

    @Override
    protected Object getConstantValue() {
        return integer;
    }
}

==> js-22.0.0.2-sources.jar: com/oracle/truffle/js/nodes/unary/IsIdenticalIntegerNode.java <==
/*
 * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.js.nodes.unary;

import com.oracle.truffle.api.dsl.Fallback;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.instrumentation.Tag;
import com.oracle.truffle.js.nodes.JavaScriptNode;
import com.oracle.truffle.js.runtime.BigInt;
import com.oracle.truffle.js.runtime.JSRuntime;

import java.util.Set;

public abstract class IsIdenticalIntegerNode extends IsIdenticalBaseNode {

    private final int integer;

    protected IsIdenticalIntegerNode(JavaScriptNode operand, int integer, boolean leftConstant) {
        super(operand, leftConstant);
        this.integer = integer;
    }

    @Specialization
    protected boolean doInt(int a) {
        return a == integer;
    }

    @Specialization
    protected boolean doDouble(double a) {
        return a == integer;
    }

    @Specialization
    protected boolean doBigInt(@SuppressWarnings("unused") BigInt a) {
        return false;
    }

    // long etc could come via Interop
    @Specialization(guards = "isJavaNumber(a)")
    protected boolean doJavaNumber(Object a) {
        double doubleValue = JSRuntime.toDouble(a);
        return doubleValue == integer;
    }

    @Fallback
    protected boolean doOther(@SuppressWarnings("unused") Object other) {
        return false;
    }

    public static IsIdenticalIntegerNode create(int integer, JavaScriptNode operand, boolean leftConstant) {
        return IsIdenticalIntegerNodeGen.create(operand, integer, leftConstant);
    }

    @Override
    protected JavaScriptNode copyUninitialized(Set<Class<? extends Tag>> materializedTags) {
        return create(integer, cloneUninitialized(getOperand(), materializedTags), leftConstant);
    }

    @Override
    protected Object getConstantValue() {
        return integer;
    }
}
